package com.agilecontrol.nea.core.web.shell;

import com.agilecontrol.nea.core.control.web.WebUtils;
import com.agilecontrol.nea.core.schema.ClientManager;
import com.agilecontrol.nea.core.util.ConfigValues;
import com.agilecontrol.nea.core.web.shell.AbstractShellCmd;
import com.agilecontrol.nea.core.web.shell.Shell;
import com.agilecontrol.nea.util.CommandExecuter;
import com.agilecontrol.nea.util.NDSException;
import com.agilecontrol.nea.util.StringUtils;
import com.agilecontrol.nea.util.Tools;
import com.agilecontrol.nea.util.Validator;
import java.io.File;
import java.text.SimpleDateFormat;
import java.util.Date;
import org.apache.commons.lang.SystemUtils;
import org.json.JSONObject;

@Shell(
   alias = "tail",
   admin = true,
   help = "tail [n] 显示server.log的最近n行信息"
)
public class Tail extends AbstractShellCmd {

   protected JSONObject process(String args, JSONObject envObj) throws Exception {
      JSONObject json = new JSONObject();
      String error = this.validateEnv(args);
      if(Validator.isNotNull(error)) {
         json.put("message", error);
         json.put("code", -1);
         return json;
      } else {
         int number = Tools.getInt(args, -1);
         String logPath = this.getServerLogPath();
         String outputFolder = ConfigValues.DIR_NEA_ROOT + File.separator + "home" + File.separator + ClientManager.getInstance().getDefaultClient().getDomain() + File.separator + this.userWeb.getUserId();
         if(!(new File(outputFolder)).exists()) {
            (new File(outputFolder)).mkdirs();
         }

         File tmpFile = new File(outputFolder + File.separator + "tail" + (new SimpleDateFormat("yyyyMMddHHmmss")).format(new Date()) + ".log");
         CommandExecuter executer = new CommandExecuter(tmpFile.getAbsolutePath());
         String command = this.tailCmd(number, logPath);
         this.logger.debug("{} > {}", command, tmpFile.getAbsolutePath());
         executer.run(command);
         this.checkFile(tmpFile.getAbsolutePath(), "Fail to create output file, check your permssion or system path for tail command");
         new JSONObject();
         StringBuilder message = new StringBuilder();
         message.append("<pre>").append(StringUtils.escapeHTMLTags(Tools.readFile(tmpFile.getPath(), "UTF-8"), false)).append("</pre>");
         json.put("message", message.toString());
         json.put("fileurl", WebUtils.getServletContext().getContextPath() + "/servlets/binserv/GetFile?filename=" + tmpFile.getName());
         json.put("target", "_blank");
         json.put("code", 0);
         return json;
      }
   }

   private String tailCmd(int lines, String serverLogPath) {
      String cmd;
      if(SystemUtils.IS_OS_WINDOWS) {
         cmd = "cmd /c tail -n " + lines + " " + serverLogPath;
      } else {
         cmd = "tail -n " + lines + " " + serverLogPath;
      }

      return cmd;
   }

   private String validateEnv(String param) throws NDSException {
      String error = null;
      if(Validator.isNull(param)) {
         error = "需要输入返回的行数，示例: tail 1000";
         return error;
      } else if(!Validator.isNumber(param)) {
         error = "需要输入返回的行数，示例: tail 1000";
         return error;
      } else {
         File logFile;
         if(SystemUtils.IS_OS_WINDOWS) {
            File logPath = new File("C:\\mksnt");
            if(!logPath.exists()) {
               error = "please install mksnt plugin by mint!";
               return error;
            }

            logFile = new File("C:\\mksnt\\tail.exe");
            if(!logFile.exists()) {
               error = "please check tail.exe in c:/mksnt ";
               return error;
            }
         }

         String logPath1 = this.getServerLogPath();
         if(logPath1 == null) {
            return "server.log not found, check ConfigValues.DIR_NEA_ROOT=" + ConfigValues.DIR_NEA_ROOT;
         } else {
            logFile = new File(logPath1);
            if(!logFile.exists()) {
               error = "Please check " + logFile.getAbsolutePath();
               return error;
            } else {
               return null;
            }
         }
      }
   }
}
